package cli;

import org.apache.commons.cli.*;

public class CommandLineInterface {

    private Options options;

    public CommandLineInterface() {
        this.options = new Options();
        addOptions();
    }


    public void addOptions() {
        OptionGroup uniqueCommands = new OptionGroup();

        Option initialiseITS = Option.builder()
                .longOpt("init")
                .desc("initialise issue system")
                .build();
        uniqueCommands.addOption(initialiseITS);

        Option createIssue = Option.builder()
                .longOpt("new")
                .hasArgs()
                .argName("TITLE> <DESCRIPTION")
                .numberOfArgs(2)
                .desc("create new issue")
                .build();
        uniqueCommands.addOption(createIssue);

        Option showIssue = Option.builder()
                .longOpt("show")
                .hasArg()
                .argName("ID")
                .desc("show issue with ID")
                .build();
        uniqueCommands.addOption(showIssue);

        Option pullIssue = Option.builder()
                .longOpt("pull")
                .hasArg()
                .argName("REMOTE")
                .desc("pull issues from remote")
                .build();
        uniqueCommands.addOption(pullIssue);

        Option pushIssues = Option.builder()
                .longOpt("push")
                .hasArg()
                .argName("REMOTE")
                .desc("push issues to remote")
                .build();
        uniqueCommands.addOption(pushIssues);

        Option ls = Option.builder()
                .longOpt("ls")
                .desc("list issues")
                .hasArgs()
                .optionalArg(true)
                .argName("ls")
                .build();

        uniqueCommands.addOption(ls);

        Option comment = Option.builder()
                .longOpt("comment")
                .hasArgs()
                .argName("ID> <MESSAGE")
                .numberOfArgs(2)
                .desc("comment on issue with given id")
                .build();
        uniqueCommands.addOption(comment);

        Option updateIssue = Option.builder()
                .longOpt("update")
                .hasArgs()
                .numberOfArgs(3)
                .argName("ID> <OPTION> <DATA")
                .desc("update issue with given id, option values: \"title\" & \"desc\" & \"status\"")
                .build();
        uniqueCommands.addOption(updateIssue);

        Option tagIssue = Option.builder()
                .longOpt("tag")
                .hasArgs()
                .numberOfArgs(3)
                .argName("ID> <OPTION> <DATA")
                .desc("add or delete tag with given id, option values: \"add\" & \"rm\"")
                .build();
        uniqueCommands.addOption(tagIssue);

        Option deleteIssue = Option.builder()
                .longOpt("delete")
                .hasArg()
                .argName("ID")
                .desc("delete issue with given id")
                .build();
        uniqueCommands.addOption(deleteIssue);

        options.addOptionGroup(uniqueCommands);
    }

    public void help() {
        HelpFormatter formatter = new HelpFormatter();
        formatter.printHelp("git ddit --<command> [args]", options);
    }

    public CommandLine getInput(String[] userInput) throws ParseException {
        CommandLineParser parser = new DefaultParser();
        return parser.parse(options, userInput);
    }
}
